import 'dart:typed_data';
import 'package:app/pages/consts.dart';
import 'package:flutter/material.dart';
import 'package:syncfusion_flutter_maps/maps.dart';
import '../consts.dart';
import 'dart:convert';
import 'dart:io';

/// A class containing thickness for each subdivision of the map.
class IceThicknessModel {
  IceThicknessModel(this.subDivID, this.thickness);

  final int subDivID;
  final double thickness;
}

/// A stateful widget which contains a choropleth map.
/// The map data is fetched from the server, and the map is rendered
/// using the Syncfusion Flutter Maps library.
class ChoroplethMap extends StatefulWidget {
  const ChoroplethMap({Key? key,}) : super(key: key);

  @override
  _ChoroplethMapState createState() => _ChoroplethMapState();
}

class _ChoroplethMapState extends State<ChoroplethMap> {
  late MapShapeSource mapShapeSource;
  List<IceThicknessModel> iceThicknessList = <IceThicknessModel>[];
  Uint8List? relation;

  @override
  void initState() {
    super.initState();
    fetchRelation(); // Fetch relation data from server
  }

  @override
  Widget build(BuildContext context) {
    return relation != null
        ? SfMaps(
        layers: [
          MapShapeLayer(
            source: MapShapeSource.memory(
              relation!,
              shapeDataField: 'name',
            ),
            color: Colors.orange,
          ),
        ],
    ) : const Center(child: CircularProgressIndicator());
  }

  // Fetch relation data from server
  Future<void> fetchRelation() async {
    try {
      // Custom HTTP client
      HttpClient client = HttpClient()
        ..badCertificateCallback = // NB: temporary disable SSL certificate validation
            (X509Certificate cert, String host, int port) => true;

      // Execute request to to get_relation endpoint
      var request = await client.getUrl(Uri.parse('${serverURI}get_relation'));
      var response = await request.close(); // Close response body at end of function

      // Parse body to JSON if request is ok
      if (response.statusCode == 200) {
        var responseBody = await response.transform(utf8.decoder).join();

        if (responseBody.isNotEmpty) {
          var jsonData = json.decode(responseBody);

          if (jsonData != null) {
            print("JSON data NOT NULL");
            // Convert JSON string to Uint8List
            setState(() {
              relation = Uint8List.fromList(utf8.encode(jsonData));
            }); // Update UI
          } else {
            throw Exception('Failed to parse response body');
          }
        } else {
          throw Exception('Response body is empty');
        }
      } else {
        throw Exception('Failed to fetch relation data: Status code ${response.statusCode}');
      }
    } catch (e) {
      throw Exception('Failed to fetch relation data: ${e.toString()}');
    }
  }
}
